/*--------------------------------------------------------------------------
 * Copyright (C) 1992 - 1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  TMCLK16.C - This utility uses TMEX to read the time from 
 *              a DS2415. It requires the Windows 3.1 TMEX
 *              drivers to be present.
 *
 * Compiler:  Microsoft Visual C 1.52
 * Version: 1.00
 *
 */

#define TMEXUTIL

#include <time.h>
#include "iBTMEXCW.H"   

/* type structure to holde time/date */
typedef struct          
{
     ushort  second;
     ushort  minute;
     ushort  hour;
     ushort  day;
     ushort  month;
     ushort  year;
} timedate;

/* Local Function Prototypes */
short FindFirstFamily(short, long);
void Exit_Prog(char *, short);
short ReadClock(long, uchar  *);
void SecondsToDate(timedate  *, uchar  *);
void DateToSeconds(timedate *, uchar *);
ulong uchar_to_bin(int, uchar  *);
short WriteClock(long, uchar *);

/* Global Variables to hold list information */
short PortType, PortNum;
uchar state_buffer[5120];
ushort CRC16;

/*----------------------------------------------------------------------
 *  This is the Main routine for SWVC.
 */
void main(short argc, char **argv)
{
   short printmsg=1,ClearActivity=0,SetFlag;
   char tstr[80];       
   char ReturnString[256];
   long session_handle;       
   timedate td;                  
   uchar time_buf[6];
   struct tm *pc_time;   
   time_t num_seconds;

   /* check arguments to see if request instruction with '?' or too many */
   if ((argc >= 2) && (argv[1][0] == '?' || argv[1][1] == '?'))
   {
       printf("\nusage: tmvc2415 <SET> \n"
              "  - read or set the time from a DS2415 \n"
              "  - <optional> argument 1 if preset will set the clock from the PC\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default PortNum and PortType from the registry */
   GetPrivateProfileString("iButton TMEX General Settings","DefaultPortNum", 
                             "1",ReturnString,255,"IBUTTON.INI");
   PortNum = atoi(ReturnString);
   GetPrivateProfileString("iButton TMEX General Settings","DefaultPortType", 
                             "1",ReturnString,255,"IBUTTON.INI");
   PortType = atoi(ReturnString);

   /* check argument to see if want first set the time from the PC */
   SetFlag = (argc > 1) ? TRUE : FALSE;

   /* check to see that indicated PortNum is  <= 15 */
   if (PortNum > 15)
     Exit_Prog("ERROR, Indicated PortNum is not valid!\n",
               -XONE_WIRE_PORT_ERROR);

   /* print a header */  
   printf("\nTMCLK16 Version 1.00\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-1999\n\n");
   printf("Default Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("TMEX Main Driver: %s\n",tstr);
   printf("TMEX Adapter Driver:");
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n\n",tstr);

   /* loop to get a session handle and read the time */
   for (;;)
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         /* setup the MicroLan */
         if (TMSetup(session_handle) == 1)
         {
            printf("Searching for a DS2415...\n\n");
            
            /* look for the first time iButton (family type 0x24) */
            if (FindFirstFamily(0x24,session_handle))
            {
               /* check if first need to set the clock */
               if (SetFlag)
               {
                  /* get the current PC time */
                  time(&num_seconds);   
                  pc_time = localtime(&num_seconds);
                  td.month = pc_time->tm_mon + 1;
                  td.day = pc_time->tm_mday;
                  if (pc_time->tm_year < 70)
                     td.year = pc_time->tm_year + 2000;
                  else
                     td.year = pc_time->tm_year + 1900;
                  td.hour = pc_time->tm_hour;
                  td.minute = pc_time->tm_min;
                  td.second = pc_time->tm_sec;

                  printf("PC time is: %02d/%02d/%04d  %2d:%02d:%02d\n\n",
                         td.month,td.day,td.year,td.hour,td.minute,
                         td.second); 

                  /* convert to seconds 
                     (could have used the initial num_seconds) */
                  DateToSeconds(&td,&time_buf[1]);

                  /* set the control byte to write */
                  time_buf[0] = 0x0C; 

                  /* write the clock and verify */
                  if (WriteClock(session_handle,time_buf) == TRUE)
                     printf("DS2415 Clock register successfully set!\n\n"); 
                  else
                  {
                     printf("Error writing DS2415 Clock register!"); 
                     break;
                  }
               }

               printf("Reading DS2415 clock register\n");

               /* read the Clock */
               if (ReadClock(session_handle,time_buf) == TRUE)
               {
                  /* convert the number of seconds to a date 
                     (could have used localtime C function) */
                  SecondsToDate(&td,&time_buf[1]);

                  /* print the control byte */
                  printf("\nControl byte read: %02X\n\n",time_buf[0]);

                  printf("Time from DS2415: %02d/%02d/%04d  %2d:%02d:%02d\n\n",
                         td.month,td.day,td.year,td.hour,td.minute,
                         td.second); 
                  break;
               }
               else
               {
                  printf("Error reading DS2415 Clock register!"); 
                  break;
               }
            }
            /* Temperature iButton not on MicroLan */
            else   
            {
               printf("DS2415 not found on MicroLan!"); 
               break;
            }
         }   
         /* MicroLan port not valid */
         else   
         {
            printf("MicroLan setup failed!"); 
            break;
         }
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   /* end the session if one is open */
   TMEndSession(session_handle);
}


/*----------------------------------------------------------------------
 * Read the time into the buffer buf.
 */                                                          
short ReadClock(long session_handle, uchar *time_buf)
{    
   short cnt=0,i,send_cnt,rslt;   
   uchar send_block[30];

   /* loop to read the time value (loop until two read the same) */
   do
   {
      /* access and verify it is there */
      if (TMAccess(session_handle,&state_buffer) == 1) 
      {
         /* create a buffer to send */
         send_cnt = 0;
         /* read clock command */
         send_block[send_cnt++] = 0x66;
         /* enought bytes to read control/clock twice */
         for (i = 0; i < 10; i++)
            send_block[send_cnt++] = 0xFF; 

         /* now send the block */
         rslt = TMBlockStream(session_handle,send_block,send_cnt);
         if (rslt == send_cnt)
         {
            /* loop to see if both reads the same */
            for (i = 1; i <= 5; i++)
               if (send_block[i] != send_block[i+5])
                  continue;

            /* must be ok, so copy to return buffer */
            for (i = 1; i <= 5; i++)
               time_buf[i - 1] = send_block[i];

            return TRUE;
         }
      }
   }
   while (++cnt < 5);

   /* failed to read the clock */
   return FALSE;
}

/*----------------------------------------------------------------------
 * Write the time from the the buffer buf.
 */                                                          
short WriteClock(long session_handle, uchar *time_buf)
{    
   short cnt=0,i,send_cnt,rslt;   
   uchar send_block[30], new_time_buf[6];
   ulong target_seconds, read_seconds;

   /* loop to read the time value (loop until two read the same) */
   do
   {
      /* access and verify it is there */
      if (TMAccess(session_handle,&state_buffer) == 1) 
      {
         /* create a buffer to send */
         send_cnt = 0;
         /* read clock command */
         send_block[send_cnt++] = 0x99;
         /* time to write include control byte */
         for (i = 0; i < 5; i++)
            send_block[send_cnt++] = time_buf[i]; 

         /* now send the block */
         rslt = TMBlockStream(session_handle,send_block,send_cnt);
         if (rslt == send_cnt)
         {
            /* send a reset to finish the operation */
            TMTouchReset(session_handle);

            /* read the clock to verify ok */
            if (ReadClock(session_handle,new_time_buf))
            {
               target_seconds = uchar_to_bin(4,&time_buf[1]);
               read_seconds = uchar_to_bin(4,&new_time_buf[1]);
               /* if within 2 seconds of target then return success */
               if ((read_seconds - target_seconds) < 2)
                  return TRUE;
            }
         }
      }
   }
   while (++cnt < 5);

   /* failed to write the clock */
   return FALSE;
}

/*--------------------------------------------------------------------------
 * Take a 4 byte long string and convert it into a timedata structure.
 */
static int dm[] = { 0,0,31,59,90,120,151,181,212,243,273,304,334,365 };

void SecondsToDate(timedate *td, uchar *seconds_buf)
{
   short tmp,i,j;
   ulong x,y;
   
   /* get value of number of seconds */
   x = uchar_to_bin(4,&seconds_buf[0]);
   
   /* check to make sure date is not over 2070 */
   if (x > 0xBBF81E00L)
      x = 0;
   
   y = x/60;  td->second = (ushort)(x-60*y);
   x = y/60;  td->minute = (ushort)(y-60*x);
   y = x/24;  td->hour   = (ushort)(x-24*y);
   x = 4*(y+731);  td->year = (ushort)(x/1461);
   i = (int)((x-1461*(ulong)(td->year))/4);  td->month = 13;
   
   do
   {
      td->month -= 1;
      tmp = (td->month > 2) && ((td->year & 3)==0) ? 1 : 0;
      j = dm[td->month]+tmp;
   
   } while (i < j);
   
   td->day = i-j+1;
   
   /* slight adjustment to algorithm */
   if (td->day == 0) td->day = 1;
   
   td->year = (td->year < 32)  ? td->year + 68 + 1900: td->year - 32 + 2000;
}

/*--------------------------------------------------------------------------
 * DateToSeconds takes a time/date structure and converts it into the 
 * 4 byte format used in DS2415.
 */
void DateToSeconds(timedate *td, uchar *seconds_buf)
{
   ulong Sv,Bv,Xv;
   int tmp;

   /* convert the date/time values into the 5 byte format used in the touch */
   if (td->year >= 2000) 
      Sv = td->year + 32 - 2000;
   else 
      Sv = td->year - 68 - 1900;

   if ((td->month > 2) && ( (Sv & 3) == 0))
     Bv = 1;
   else
     Bv = 0;

   Xv = 365 * (Sv-2) + (Sv-1)/4 + dm[td->month] + td->day + Bv - 1;

   Xv = 86400 * Xv + (ulong)(td->second) + 60*((ulong)(td->minute) + 60*(ulong)(td->hour));

   /* copy the result into the seconds buffer */
   for (tmp = 0; tmp < 4; tmp++)
   {
       seconds_buf[tmp] = (uchar)(Xv & 0xFF);
       Xv >>= 8;
   }
}



/*--------------------------------------------------------------------------
 * UCHAR_TO_BIN converts a binary uchar string 'str' into a ulong return
 * number.  'num' indicates the length of the binary uchar string.
 */
ulong uchar_to_bin(int num, uchar *str)
{
   short i;
   ulong l = 0;
      
   for (i = (num-1); i >= 0; i--)
      l = (l << 8) | (int)(str[i]);
      
   return l;
}
                               

/*----------------------------------------------------------------------
 * Find the first device with a particular family
 */
short FindFirstFamily(short family, long session_handle)
{
   short j;
   short ROM[9];
   
   /* set up to find the first device with family 'family */
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      /* get first device in list with the specified family */
      if (TMNext(session_handle,state_buffer) == 1)
      {  
         /* read the rom number */
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         /* check if correct type */
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            printf("DS2415 found with ID: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("\n\n");
            return 1;
         }        
      }
   }
     
   /* failed to find device of that type */
   return 0;                              
}

/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
   /* print the message left justified */
   printf("\r%s",msg);
   
   /*  error beep if return code is not 0 */
   if (errcd != NORMAL_EXIT)
     MessageBeep(MB_ICONEXCLAMATION);
   
   /*  return error code passed */
   exit(errcd);
}

                               
